from PyaiVS.data_utils import TVT,statistical
import warnings
from PyaiVS.splitdater import split_dataset
from PyaiVS.feature_create import create_des
from hyperopt import fmin, tpe, hp, STATUS_OK, Trials
import joblib
from sklearn.metrics import roc_auc_score, mean_squared_error,r2_score, mean_absolute_error

import pandas as pd
import numpy as np
from sklearn.ensemble import RandomForestClassifier
from sklearn.ensemble import RandomForestRegressor
def all_one_zeros(data):
    if (len(np.unique(data)) == 2):
        flag = False
    else:
        flag = True
    return flag


warnings.filterwarnings("ignore")

# the metrics for classification

def best_model_runing(seed,best_hyper,data,split_type='random',FP_type='ECFP4',task_type ='cla',model_dir=False):
    pd_res = []

    if task_type == 'cla':
        while True:

            # data_tr_x, data_va_x, data_te_x, data_tr_y, data_va_y, data_te_y = split_dataset(X, Y,
            #                                                                                      split_type=split_type,
            #                                                                                      valid_need=True,
            #                                                                                      random_state=seed)
            data_x, data_te_x, data_y, data_te_y = data.set2ten(seed)
            data_tr_x, data_va_x, data_tr_y, data_va_y = split_dataset(data_x, data_y, split_type='random', valid_need=False,
                                                                       random_state=seed, train_size=(8 / 9))

            data_tr_x, data_tr_y = create_des(data_tr_x, data_tr_y, FP_type=FP_type,model_dir=model_dir)
            data_va_x, data_va_y = create_des(data_va_x, data_va_y, FP_type=FP_type,model_dir=model_dir)
            data_te_x, data_te_y = create_des(data_te_x, data_te_y, FP_type=FP_type,model_dir=model_dir)
            if (all_one_zeros(data_tr_y) or all_one_zeros(data_va_y) or all_one_zeros(data_te_y)):
                # print(
                #     '\ninvalid random seed {} due to one class presented in the {} splitted sets...'.format(seed,
                #                                                                                             split_type))
                # print('Changing to another random seed...\n')
                seed = np.random.randint(50, 999999)
            else:

                break
    else:

        data_x, data_te_x, data_y, data_te_y = data.set2ten(seed)
        data_tr_x, data_va_x, data_tr_y, data_va_y = split_dataset(data_x, data_y, split_type='random',
                                                                   valid_need=False,
                                                                   random_state=seed, train_size=(8 / 9))
        data_tr_x, data_tr_y = create_des(data_tr_x, data_tr_y, FP_type=FP_type)
        data_va_x, data_va_y = create_des(data_va_x, data_va_y, FP_type=FP_type)
        data_te_x, data_te_y = create_des(data_te_x, data_te_y, FP_type=FP_type)

    model = RandomForestClassifier(n_estimators= best_hyper['n_estimators'],
                                   max_depth=best_hyper['max_depth'],
                                   min_samples_leaf=best_hyper['min_samples_leaf'],
                                   max_features=best_hyper['max_features'],

                                   n_jobs=6, random_state=1, verbose=0, class_weight='balanced') \
        if task_type == 'cla' else RandomForestRegressor(
        n_estimators=best_hyper['n_estimators'],
        max_depth=best_hyper['max_depth'],
        min_samples_leaf=best_hyper['min_samples_leaf'],
        max_features=best_hyper['max_features'],

        n_jobs=6, random_state=1, verbose=0)

    model.fit(data_tr_x, data_tr_y)
    num_of_compounds = data_x.shape[0]+data_te_x.shape[0]
    if task_type == 'cla':
        tr_pred = model.predict_proba(data_tr_x)
        tr_results = [seed, FP_type, split_type, 'tr', num_of_compounds]
        tr_results.extend(statistical(data_tr_y, np.argmax(tr_pred, axis=1), tr_pred[:, 1]))
        pd_res.append(tr_results)
        # validation set
        va_pred = model.predict_proba(data_va_x)
        va_results = [seed, FP_type, split_type, 'va', num_of_compounds]
        va_results.extend(statistical(data_va_y, np.argmax(va_pred, axis=1), va_pred[:, 1]))
        pd_res.append(va_results)
        # test set
        te_pred = model.predict_proba(data_te_x)
        te_results = [seed, FP_type, split_type, 'te', num_of_compounds]
        te_results.extend(statistical(data_te_y, np.argmax(te_pred, axis=1), te_pred[:, 1]))
        pd_res.append(te_results)
    else:
        # training set
        tr_pred = model.predict(data_tr_x)
        tr_results = [seed, FP_type, split_type, 'tr', num_of_compounds,
                      np.sqrt(mean_squared_error(data_tr_y, tr_pred)), r2_score(data_tr_y, tr_pred),
                      mean_absolute_error(data_tr_y, tr_pred)]
        pd_res.append(tr_results)
        # validation set
        va_pred = model.predict(data_va_x)
        va_results = [seed, FP_type, split_type, 'va', num_of_compounds,
                      np.sqrt(mean_squared_error(data_va_y, va_pred)), r2_score(data_va_y, va_pred),
                      mean_absolute_error(data_va_y, va_pred)]
        pd_res.append(va_results)
        # test set
        te_pred = model.predict(data_te_x)
        te_results = [seed, FP_type, split_type, 'te', num_of_compounds,
                      np.sqrt(mean_squared_error(data_te_y, te_pred)), r2_score(data_te_y, te_pred),
                      mean_absolute_error(data_te_y, te_pred)]
        pd_res.append(te_results)

    return pd_res
def tvt_rf(X,Y,split_type='random',FP_type='ECFP4',task_type='cla',model_dir=False):
    random_state = 42
    while True:

        # data_tr_x, data_va_x, data_te_x, data_tr_y, data_va_y, data_te_y = split_dataset(X, Y,
        #                                                                                  split_type=split_type,
        #                                                                                  valid_need=True,
        #                                                                                  random_state=random_state)
        data = TVT(X,Y)
        data_x, data_te_x, data_y, data_te_y = data.set2ten(0)
        data_tr_x, data_va_x, data_tr_y, data_va_y = split_dataset(data_x, data_y, split_type=split_type, valid_need=False,
                                                                   random_state=random_state, train_size=(8 / 9))
        data_tr_x, data_tr_y = create_des(data_tr_x, data_tr_y, FP_type=FP_type, model_dir=model_dir)
        data_va_x, data_va_y = create_des(data_va_x, data_va_y, FP_type=FP_type, model_dir=model_dir)
        data_te_x, data_te_y = create_des(data_te_x, data_te_y, FP_type=FP_type, model_dir=model_dir)

        if (all_one_zeros(data_tr_y) or all_one_zeros(data_va_y) or all_one_zeros(data_te_y)):
            # print(
            #     '\ninvalid random seed {} due to one class presented in the {} splitted sets...'.format('None',
            #                                                                                             split_type))
            #
            random_state += np.random.randint(50, 999999)
            #
            # print('Changing to another random seed {}\n'.format(random_state))
        else:


            break

    pd_res = []
    OPT_ITERS = 50

    space_ = {'n_estimators': hp.choice('n_estimators', [10, 50, 100, 200, 300, 400, 500]),
              'max_depth': hp.choice('max_depth', range(3, 12)),
              'min_samples_leaf': hp.choice('min_samples_leaf', [1, 3, 5, 10, 20, 50]),

              'max_features': hp.choice('max_features', ['sqrt', 'log2', 0.7, 0.8, 0.9])
              }
    n_estimators_ls = [10, 50, 100, 200, 300, 400, 500]
    max_depth_ls = range(3, 12)
    min_samples_leaf_ls = [1, 3, 5, 10, 20, 50]
    max_features_ls = ['sqrt', 'log2', 0.7, 0.8, 0.9]

    trials = Trials()

    def hyper_opt(args):
        model = RandomForestClassifier(**args, n_jobs=6, random_state=1, verbose=0, class_weight='balanced') \
            if task_type == 'cla' else RandomForestRegressor(**args, n_jobs=6, random_state=1, verbose=0)
        model.fit(data_tr_x, data_tr_y)
        val_preds = model.predict_proba(data_va_x) if task_type == 'cla' else model.predict(data_va_x)
        loss = 1 - roc_auc_score(data_va_y, val_preds[:, 1]) if task_type == 'cla' else np.sqrt(
            mean_squared_error(data_va_y, val_preds))
        return {'loss': loss, 'status': STATUS_OK}


    # start hyper-parameters optimization
    best_results = fmin(hyper_opt, space_, algo=tpe.suggest, max_evals=OPT_ITERS, trials=trials, show_progressbar=False)

    # best_model = RandomForestClassifier(n_estimators=n_estimators_ls[best_results['n_estimators']],
    #                                     max_depth=max_depth_ls[best_results['max_depth']],
    #                                     min_samples_leaf=min_samples_leaf_ls[best_results['min_samples_leaf']],
    #                                     max_features=max_features_ls[best_results['max_features']],
    #
    #                                     n_jobs=6, random_state=1, verbose=0, class_weight='balanced') \
    #     if task_type == 'cla' else RandomForestRegressor(
    #     n_estimators=n_estimators_ls[best_results['n_estimators']],
    #     max_depth=max_depth_ls[best_results['max_depth']],
    #     min_samples_leaf=min_samples_leaf_ls[best_results['min_samples_leaf']],
    #     max_features=max_features_ls[best_results['max_features']],
    #
    #     n_jobs=6, random_state=1, verbose=0)
    # best_model.fit(data_tr_x, data_tr_y)
    # if model_dir :
    #     model_name = str(model_dir) +'/%s_%s_%s_%s'%(split_type,task_type,FP_type,'RF_bestModel.pkl')
    #     joblib.dump(best_model,model_name)
    # num_of_compounds = len(X)
    # if task_type == 'cla':
    #     tr_pred = best_model.predict_proba(data_tr_x)
    #     tr_results = [FP_type, split_type, 'tr', num_of_compounds,
    #                   n_estimators_ls[best_results['n_estimators']],
    #                   max_depth_ls[best_results['max_depth']],
    #                   min_samples_leaf_ls[best_results['min_samples_leaf']],
    #
    #                   max_features_ls[best_results['max_features']]]
    #     tr_results.extend(statistical(data_tr_y, np.argmax(tr_pred, axis=1), tr_pred[:, 1]))
    #     pd_res.append(tr_results)
    #     # validation set
    #     va_pred = best_model.predict_proba(data_va_x)
    #     va_results = [FP_type, split_type, 'va', num_of_compounds,
    #                   n_estimators_ls[best_results['n_estimators']],
    #                    max_depth_ls[best_results['max_depth']],
    #                    min_samples_leaf_ls[best_results['min_samples_leaf']],
    #
    #                    max_features_ls[best_results['max_features']]]
    #     va_results.extend(statistical(data_va_y, np.argmax(va_pred, axis=1), va_pred[:, 1]))
    #     pd_res.append(va_results)
    #     # test set
    #     te_pred = best_model.predict_proba(data_te_x)
    #     te_results = [FP_type, split_type, 'te', num_of_compounds,
    #                   n_estimators_ls[best_results['n_estimators']],
    #                    max_depth_ls[best_results['max_depth']],
    #                    min_samples_leaf_ls[best_results['min_samples_leaf']],
    #
    #                    max_features_ls[best_results['max_features']]]
    #     te_results.extend(statistical(data_te_y, np.argmax(te_pred, axis=1), te_pred[:, 1]))
    #     pd_res.append(te_results)
    #     para_res = pd.DataFrame(pd_res, columns=['FP_type', 'split_type', 'type','num_of_compounds',
    #             'n_estimators', 'max_depth','min_samples_leaf','max_features',
    #             'precision', 'se', 'sp',
    #             'acc', 'mcc', 'auc_prc', 'auc_roc'])
    # else:
    #     # training set
    #     tr_pred = best_model.predict(data_tr_x)
    #     tr_results = [FP_type, split_type, 'va', num_of_compounds,
    #                   n_estimators_ls[best_results['n_estimators']],
    #                   max_depth_ls[best_results['max_depth']],
    #                   min_samples_leaf_ls[best_results['min_samples_leaf']],
    #
    #                   max_features_ls[best_results['max_features']],
    #                   np.sqrt(mean_squared_error(data_tr_y, tr_pred)), r2_score(data_tr_y, tr_pred),
    #                   mean_absolute_error(data_tr_y, tr_pred)]
    #     pd_res.append(tr_results)
    #     # validation set
    #     va_pred = best_model.predict(data_va_x)
    #     va_results = [FP_type, split_type, 'va', num_of_compounds,
    #                   n_estimators_ls[best_results['n_estimators']],
    #                   max_depth_ls[best_results['max_depth']],
    #                   min_samples_leaf_ls[best_results['min_samples_leaf']],
    #
    #                   max_features_ls[best_results['max_features']],
    #                   np.sqrt(mean_squared_error(data_va_y, va_pred)), r2_score(data_va_y, va_pred),
    #                   mean_absolute_error(data_va_y, va_pred)]
    #     pd_res.append(va_results)
    #     # test set
    #     te_pred = best_model.predict(data_te_x)
    #     te_results = [FP_type, split_type, 'va', num_of_compounds,
    #                   n_estimators_ls[best_results['n_estimators']],
    #                   max_depth_ls[best_results['max_depth']],
    #                   min_samples_leaf_ls[best_results['min_samples_leaf']],
    #
    #                   max_features_ls[best_results['max_features']],
    #                   np.sqrt(mean_squared_error(data_te_y, te_pred)), r2_score(data_te_y, te_pred),
    #                   mean_absolute_error(data_te_y, te_pred)]
    #     pd_res.append(te_results)
    #     para_res = pd.DataFrame(pd_res, columns=['FP_type', 'split_type', 'type', 'num_of_compounds',
    #                                              'n_estimators', 'max_depth', 'min_samples_leaf',
    #                                               'max_features',
    #                                              'rmse', 'r2', 'mae'])
    n_estimators, max_depth, min_samples_leaf,max_features= n_estimators_ls[best_results['n_estimators']],\
            max_depth_ls[best_results['max_depth']],min_samples_leaf_ls[best_results['min_samples_leaf']],max_features_ls[best_results['max_features']]
    best_results = {'n_estimators':n_estimators, 'max_depth':max_depth, "min_samples_leaf":min_samples_leaf, 'max_features':max_features}
    # print(best_results)
    para_file = str(model_dir).replace('model_save', 'param_save') + '/%s_%s_%s_%s' % (
    split_type, task_type, FP_type, 'RF.param')
    import os
    if not os.path.exists(str(model_dir).replace('model_save', 'param_save')):
        os.makedirs(str(model_dir).replace('model_save', 'param_save'))
    # print(os.path.exists(str(model_dir).replace('model_save', 'param_save')))
    f = open(para_file, 'w')
    f.write('%s' % best_results)
    f.close()
    # pd_res = []
    # # for i in range(10):
    # #     item = best_model_runing((i+1)*500,best_results,X,Y,split_type=split_type,FP_type=FP_type,task_type=task_type,model_dir=model_dir)
    # for i in range(9):
    #     item = best_model_runing((i + 1) , best_results, data, split_type=split_type, FP_type=FP_type,
    #                                  task_type=task_type, model_dir=model_dir)
    #
    #     pd_res.extend(item)
    #
    # if task_type=='cla':
    #     best_res = pd.DataFrame(pd_res, columns=['seed','FP_type', 'split_type', 'type',
    #                                                      'num_of_compounds', 'precision', 'se', 'sp',
    #                                                      'acc', 'mcc', 'auc_prc', 'auc_roc'])
    #
    #     pd1 = para_res[['FP_type', 'split_type', 'type',
    #                                                      'num_of_compounds', 'precision', 'se', 'sp',
    #                                                      'acc', 'mcc', 'auc_prc', 'auc_roc']]
    #     pd1['seed'] =0
    #     best_res = pd.concat([pd1,best_res],ignore_index=True)
    #     best_res = best_res.groupby(['FP_type', 'split_type', 'type'])['acc', 'mcc', 'auc_prc', 'auc_roc'].mean()
    # else:
    #     best_res = pd.DataFrame(pd_res, columns=['seed', 'FP_type', 'split_type', 'type',
    #                                              'num_of_compounds','rmse', 'r2', 'mae'])
    #     pd1 = para_res[['FP_type', 'split_type', 'type',
    #                             'num_of_compounds','rmse', 'r2', 'mae']]
    #     pd1['seed'] = 0
    #     best_res = pd.concat([pd1, best_res], ignore_index=True)
    # return  para_res,best_res

def para_rf(X,Y,args=None,split_type='random',FP_type='ECFP4',task_type = 'cla',model_dir=None):
    param_file = str(model_dir).replace('model_save', 'param_save') + '/%s_%s_%s_%s' % (
    split_type, task_type, FP_type, 'RF.param')
    args = eval(open(param_file, 'r').readline().strip()) if args == None else args
    data = TVT(X, Y)
    data_x, data_te_x, data_y, data_te_y = data.set2ten(0)
    data_tr_x, data_va_x, data_tr_y, data_va_y = split_dataset(data_x, data_y, split_type=split_type, valid_need=False,
                                                               random_state=42, train_size=(8 / 9))
    data_tr_x, data_tr_y = create_des(data_tr_x, data_tr_y, FP_type=FP_type, model_dir=model_dir)
    data_va_x, data_va_y = create_des(data_va_x, data_va_y, FP_type=FP_type, model_dir=model_dir)
    data_te_x, data_te_y = create_des(data_te_x, data_te_y, FP_type=FP_type, model_dir=model_dir)

    pd_res = []
    n_estimators = args['n_estimators']
    max_depth = args['max_depth']
    min_samples_leaf = args['min_samples_leaf']
    max_features = args['max_features']



    best_model = RandomForestClassifier(n_estimators=n_estimators,
                                        max_depth=max_depth,
                                        min_samples_leaf=min_samples_leaf,
                                        max_features=max_features,

                                        n_jobs=6, random_state=1, verbose=0, class_weight='balanced') \
        if task_type == 'cla' else RandomForestRegressor(
        n_estimators=n_estimators,
        max_depth=max_depth,
        min_samples_leaf=min_samples_leaf,
        max_features=max_features,

        n_jobs=6, random_state=1, verbose=0)
    best_model.fit(data_tr_x, data_tr_y)
    if model_dir :
        model_name = str(model_dir) +'/%s_%s_%s_%s'%(split_type,task_type,FP_type,'RF_bestModel.pkl')
        joblib.dump(best_model,model_name)
    num_of_compounds = len(X)
    if task_type == 'cla':
        tr_pred = best_model.predict_proba(data_tr_x)
        tr_results = [FP_type, split_type, 'tr', num_of_compounds,
                      n_estimators,max_depth,min_samples_leaf,max_features]
        tr_results.extend(statistical(data_tr_y, np.argmax(tr_pred, axis=1), tr_pred[:, 1]))
        pd_res.append(tr_results)
        # validation set
        va_pred = best_model.predict_proba(data_va_x)
        va_results = [FP_type, split_type, 'va', num_of_compounds,
                      n_estimators,max_depth,min_samples_leaf,max_features]
        va_results.extend(statistical(data_va_y, np.argmax(va_pred, axis=1), va_pred[:, 1]))
        pd_res.append(va_results)
        # test set
        te_pred = best_model.predict_proba(data_te_x)
        te_results = [FP_type, split_type, 'te', num_of_compounds,
                      n_estimators,max_depth,min_samples_leaf,max_features]
        te_results.extend(statistical(data_te_y, np.argmax(te_pred, axis=1), te_pred[:, 1]))
        pd_res.append(te_results)
        para_res = pd.DataFrame(pd_res, columns=['FP_type', 'split_type', 'type', 'num_of_compounds',
                                                 'n_estimators','max_depth','min_samples_leaf','max_features',
                                                 'precision', 'se', 'sp',
                                                 'acc', 'mcc', 'auc_prc', 'auc_roc'])
    else:
        # training set
        tr_pred = best_model.predict(data_tr_x)
        tr_results = [FP_type, split_type, 'tr',  num_of_compounds,
                      n_estimators,max_depth,min_samples_leaf,max_features,
                      np.sqrt(mean_squared_error(data_tr_y, tr_pred)), r2_score(data_tr_y, tr_pred),
                      mean_absolute_error(data_tr_y, tr_pred)]
        pd_res.append(tr_results)
        # validation set
        va_pred = best_model.predict(data_va_x)
        va_results = [FP_type, split_type,'va',  num_of_compounds,
                      n_estimators,max_depth,min_samples_leaf,max_features,
                      np.sqrt(mean_squared_error(data_va_y, va_pred)), r2_score(data_va_y, va_pred),
                      mean_absolute_error(data_va_y, va_pred)]
        pd_res.append(va_results)
        # test set
        te_pred = best_model.predict(data_te_x)
        te_results = [FP_type, split_type, 'te',  num_of_compounds,
                      n_estimators,max_depth,min_samples_leaf,max_features,
                      np.sqrt(mean_squared_error(data_te_y, te_pred)), r2_score(data_te_y, te_pred),
                      mean_absolute_error(data_te_y, te_pred)]
        pd_res.append(te_results)
        para_res = pd.DataFrame(pd_res, columns=['FP_type', 'split_type', 'type', 'num_of_compounds',
                                                 'n_estimators','max_depth','min_samples_leaf','max_features',
                                                 'rmse', 'r2', 'mae'])
    pd_res = []
    for i in range(9):
        item = best_model_runing((i + 1), args, data, split_type=split_type, FP_type=FP_type,
                                     task_type=task_type, model_dir=model_dir)
        pd_res.extend(item)

    if task_type == 'cla':
        best_res = pd.DataFrame(pd_res, columns=['seed', 'FP_type', 'split_type', 'type',
                                                     'num_of_compounds', 'precision', 'se', 'sp',
                                                     'acc', 'mcc', 'auc_prc', 'auc_roc'])
        pd1 = para_res[['FP_type', 'split_type', 'type',
                            'num_of_compounds', 'precision', 'se', 'sp',
                            'acc', 'mcc', 'auc_prc', 'auc_roc']]
        pd1['seed'] = 0
        best_res = pd.concat([pd1, best_res], ignore_index=True)
        # best_res = best_res.groupby(['FP_type', 'split_type', 'type'])['acc', 'mcc', 'auc_prc', 'auc_roc'].mean()
    else:
        best_res = pd.DataFrame(pd_res, columns=['seed', 'FP_type', 'split_type', 'type',
                                                     'num_of_compounds', 'rmse', 'r2', 'mae'])
        pd1 = para_res[['FP_type', 'split_type', 'type',
                            'num_of_compounds', 'rmse', 'r2', 'mae']]
        pd1['seed'] = 0
        best_res = pd.concat([pd1, best_res], ignore_index=True)
    import os
    result_dir = model_dir.replace('model_save', 'result_save')
    para_name, best_name = os.path.join(result_dir,
                                        '_'.join([split_type,'RF' , FP_type, 'para.csv'])), os.path.join(
        result_dir, '_'.join([split_type, 'RF', FP_type, 'best.csv']))
    para_res.to_csv(para_name, index=False)
    best_res.to_csv(best_name, index=False)

# df = pd.read_csv('/data/jianping/bokey/OCAICM/dataset/aurorab/aurorab.csv')
# # args = {'n_estimators': 300, 'max_depth': 11, 'min_samples_leaf': 3, 'max_features': 0.8}
# para_rf(df['Smiles'],df['activity'],split_type='scaffold',FP_type='MACCS',task_type = 'cla',model_dir='/data/jianping/bokey/OCAICM/dataset/aurorab/model_save/RF')
# # a,b =tvt_rf(df['Smiles'],df['activity'],split_type='scaffold',FP_type='MACCS',task_type = 'cla',model_dir=False)
# # print(a,b)




import warnings
import numpy as np
import pandas as pd
import argparse
import random as rn
import multiprocessing as mp
import time
import os
import sys


from PyaiVS.DNN_model import para_dnn
from PyaiVS.graph_model import para_dl
import itertools

np.set_printoptions(threshold=sys.maxsize)
# Random seeding
os.environ['PYTHONHASHSEED'] = '0'
np.random.seed(123)
rn.seed(123)
warnings.filterwarnings('ignore')


def file_merge(file_df, models):
    filelist = {}
    dataset = file_df.split('/')[-1].replace('.csv', '')
    df = pd.read_csv(file_df)
    cols = list(df.columns)
    cols.remove('Smiles')
    path = './dataset/{0}/result_save'.format(dataset)
    for task in cols:
        for model in models:
            for file in os.listdir(os.path.join(os.path.join(path, task), model)):
                if 'para' in file:
                    continue
                filelist.setdefault(file, []).append(
                    os.path.join(os.path.join(os.path.join(os.path.join(path, task), model)), file))
    for rtype in filelist.keys():
        rlist = filelist[rtype]
        mer = pd.DataFrame()
        for file in rlist:
            df = pd.read_csv(file)

            mer = pd.concat([mer, df], ignore_index=True)
        mer = mer.groupby(['seed', 'FP_type', 'split_type', 'type'])[
            ['se', 'sp', 'acc', 'mcc', 'precision', 'auc_prc', 'auc_roc']].mean()
        mer = mer.reset_index()
        save_path = os.path.join(path, rtype.split('_')[1])

        if not os.path.exists(save_path):
            os.makedirs(save_path)

        abspath = os.path.join(save_path, rtype)
        mer.to_csv(abspath, index=False)
    pass


def model_set(X, Y, split_type='random', FP_type='ECFP4', model_type='SVM', model_dir=False, file_name=None,
              task_type='cla', difftasks=None):
    if model_type == 'DNN':
        para, best = para_dnn(X, Y, split_type=split_type, FP_type=FP_type, model_dir=model_dir, file_name=file_name,
                             difftasks=difftasks)

    elif model_type == 'gat':
        para, best = para_dl(X, Y, split_type=split_type, file_name=file_name, model_name='gat', model_dir=model_dir,
                            difftasks=difftasks, task_type=task_type, device='cpu')
    elif model_type == 'attentivefp':
        para, best = para_dl(X, Y, split_type=split_type, file_name=file_name, model_name='attentivefp',
                            model_dir=model_dir, difftasks=difftasks, task_type=task_type, device='cpu')
    elif model_type == 'gcn':
        para, best = para_dl(X, Y, split_type=split_type, file_name=file_name, model_name='gcn', model_dir=model_dir,
                            difftasks=difftasks, task_type=task_type, device='cpu')
    else:
        para, best = para_dl(X, Y, split_type=split_type, file_name=file_name, model_name='mpnn', model_dir=model_dir,
                            difftasks=difftasks, task_type=task_type, device='cpu')
    return para, best


def pair_param(file_name, data, split_type, model_type, FP_type, difftasks):
    if model_type in ['DNN']:
        X = data.Smiles
        Y = data[difftasks[0]]
        model_path = 'model_save/' + model_type
        model_dir = file_name.replace(file_name.split('/')[-1], model_path)

        if not os.path.exists(model_dir):
            os.makedirs(model_dir)
        result_path = 'result_save/' + model_type
        result_dir = file_name.replace(file_name.split('/')[-1], result_path)
        if not os.path.exists(result_dir):
            os.makedirs(result_dir)

        para_name, best_name = os.path.join(result_dir, '_'.join([split_type, model_type, FP_type,'para.csv'])), os.path.join(
            result_dir, '_'.join([split_type, FP_type,model_type, 'best.csv']))
        if os.path.exists(para_name):

            pass
        else:
            para, best = model_set(X, Y, split_type=split_type, model_type=model_type, model_dir=model_dir,
                                   file_name=file_name, difftasks=difftasks)
            para.to_csv(para_name, index=False)
            best.to_csv(best_name, index=False)


    else:
        X = data.Smiles
        Y = data[difftasks[0]]
        model_path = 'model_save/' + model_type
        model_dir = file_name.replace(file_name.split('/')[-1], model_path)
        result_path = 'result_save/' + model_type
        result_dir = file_name.replace(file_name.split('/')[-1], result_path)
        if not os.path.exists(model_dir):
            os.makedirs(model_dir)
        if not os.path.exists(result_dir):
            os.makedirs(result_dir)
        para_name, best_name = os.path.join(result_dir, '_'.join([split_type, model_type, 'para.csv'])), os.path.join(
            result_dir, '_'.join([split_type, model_type, 'best.csv']))
        para, best = model_set(X, Y, split_type=split_type, model_type=model_type, model_dir=model_dir,
                                   file_name=file_name, difftasks=difftasks)
        para.to_csv(para_name, index=False)
        best.to_csv(best_name, index=False)

        return
    return


def parse_args():
    parser = argparse.ArgumentParser()
    parser.add_argument('--file', required=True, help='we must give this para')
    parser.add_argument('--split', default=['scaffold'], nargs='*', choices=['random', 'scaffold', 'cluster'])
    parser.add_argument('--FP', default=['MACCS'], nargs='*', choices=['ECFP4', 'MACCS', '2d-3d', 'pubchem'])
    parser.add_argument('--model', default=['SVM'], nargs='*', choices=['DNN', 'KNN', 'SVM', 'RF', 'XGB'])
    parser.add_argument('--threads', default=10, type=int)
    parser.add_argument('--mpl', default=False, type=str)
    parser.add_argument('--device', default='cpu', choices=['cpu', 'gpu'])
    args = parser.parse_args()
    return args


def main(file, split, FP, models, cpus, mpl, device):
    ml_models = [model for model in models if model in ['KNN', 'SVM', 'RF', 'XGB']]
    ml = {'split': split, 'FP': FP, 'model': ml_models}
    ml = pd.DataFrame(list(itertools.product(*ml.values())), columns=ml.keys())
    dl_models = [model for model in models if model in ['gcn', 'gat', 'mpnn', 'attentivefp']]
    dl = {'split': split, 'model': dl_models}
    dl = pd.DataFrame(list(itertools.product(*dl.values())), columns=dl.keys())
    data = pd.read_csv(file, error_bad_lines=False)
    difftasks = list(data.columns)
    difftasks.remove('Smiles')

    if mpl == False:
        for i in range(len(ml.index.values)):
            start_time = time.time()
            split_type = ml.iloc[i].split
            FP_type = ml.iloc[i].FP
            model_type = ml.iloc[i].model
            a = pair_param(file, data, split_type, model_type, FP_type, difftasks)
            use_time = round(time.time() - start_time, 5)
            if use_time >= 1:
                local_time = time.strftime('%Y-%m-%d %H:%M:%S', time.localtime())
                record_save = file.replace(file.split('/')[-1], 'record_every_model.csv')
                f = open(record_save, 'a+')
                f.write(",".join([file, split_type, model_type, str(use_time), local_time, FP_type, '\n']))
                f.close()
    else:
        p = mp.Pool(processes=cpus)
        for i in range(len(ml.index.values)):
            start_time = time.time()
            split_type = ml.iloc[i].split
            FP_type = ml.iloc[i].FP
            model_type = ml.iloc[i].model
            result = p.apply_async(pair_param, args=(file, data, split_type, model_type, FP_type, difftasks))
            use_time = int(time.time() - start_time)
            if use_time >= 1:
                local_time = time.strftime('%Y-%m-%d %H:%M:%S', time.localtime())
                record_save = file.replace(file.split('/')[-1], 'record_every_model.csv')
                f = open(record_save, 'a+')
                f.write(",".join([file, split_type, model_type, str(use_time), local_time, FP_type, '\n']))
                f.close()
        p.close()
        p.join()
    if 'DNN' in models:
        dnn = {'split': split, 'FP': FP, 'model': ['DNN']}
        dnn = pd.DataFrame(list(itertools.product(*dnn.values())), columns=dnn.keys())
        for i in range(len(dnn.index.values)):
            start_time = time.time()
            split_type = dnn.iloc[i].split
            FP_type = dnn.iloc[i].FP
            model_type = dnn.iloc[i].model
            a = pair_param(file, data, split_type, model_type, FP_type, difftasks)
            use_time = round(time.time() - start_time, 5)
            local_time = time.strftime('%Y-%m-%d %H:%M:%S', time.localtime())
            if use_time >= 1:
                record_save = file.replace(file.split('/')[-1], 'record_every_model.csv')
                f = open(record_save, 'a+')
                f.write(",".join([file, split_type, model_type, str(use_time), local_time, FP_type, '\n']))
                f.close()
    for i in range(len(dl.index.values)):
        split_type = dl.iloc[i].split
        model_type = dl.iloc[i].model
        start_time = time.time()
        a = pair_param(file, data, split_type, model_type, device, difftasks)
        use_time = int(time.time() - start_time)
        if use_time >= 1:
            local_time = time.strftime('%Y-%m-%d %H:%M:%S', time.localtime())
            record_save = file.replace(file.split('/')[-1], 'record_every_model.csv')
            f = open(record_save, 'a+')
            f.write(",".join([file, split_type, model_type, str(use_time), local_time, device, "\n"]))
            f.close()

    if len(difftasks) > 1:
        if len(set((['KNN', 'SVM', 'RF', 'XGB'])) | set(models)) > 0:
            need = list(set((['KNN', 'SVM', 'RF', 'XGB'])) & set(models))
            file_merge(file, need)


if __name__ == '__main__':
    # args = parse_args()
    # file = args.file
    # split = args.split
    # FP = args.FP
    # model = args.model
    # cpus = args.threads
    # mpl = args.mpl
    # device = args.device
    # main(file, split, FP, model, cpus, mpl, device)
    # running('/data/jianping/bokey/OCAICM/dataset/aurorab/aurorab.csv',out_dir='/data/jianping/bokey/OCAICM/dataset/',split='all',model=['SVM','KNN','XGB','RF','DNN'],FP='all', cpus=8)
    main('/data/jianping/bokey/OCAICM/dataset/aurorab/aurorab.csv',['scaffold'],['ECFP4', 'MACCS'],['gcn'],8,False,'cpu')


import os
import multiprocessing as mp
from PyaiVS import ml_screener
from PyaiVS import dl_screener
from PyaiVS import dnn_screener
def model_screen(model=None,FP= None,split=None,screen_file=None,prop = 0.5,sep = ',',model_dir=None,smiles_col =None):
    split = split if type(split) != type(None) else 'random'
    model = model if type(model) != type(None) else 'SVM'
    FP = FP if type(FP) != type(None) else 'MACCS'
    FP_list = ['2d-3d', 'MACCS', 'ECFP4', 'pubchem', 'gcn', 'gat', 'attentivefp', 'mpnn']
    split_list = ['random', 'scaffold', 'cluster']
    model_list = ['SVM', 'KNN', 'DNN', 'RF', 'XGB', 'gcn', 'gat', 'attentivefp', 'mpnn']
    assert len(set(list([split])) - set(split_list)) == 0, '{} element need in {}'.format(split, split_list)
    assert len(set(list([model])) - set(model_list)) == 0, '{} element need in {}'.format(model, model_list)
    assert len(set(list([FP])) - set(FP_list)) == 0, '{} element need in {}'.format(FP, FP_list)
    assert os.path.exists(model_dir),'no such model_dir {}'.format(model_dir)
    assert len(set(list([screen_file.split('.')[-1]])) - set(
        list(['csv', 'txt', 'tsv']))) == 0, '{} need in ["csv","txt","tsv"]'.format(screen_file.split('.')[-1])
    if model in ['SVM', 'KNN',  'RF', 'XGB']:
        model_file = '_'.join([split,'cla',FP,model,'bestModel.pkl'])
        model_path = os.path.join(os.path.join(model_dir,model),model_file)
        assert os.path.exists(model_path),'no such model_path {}'.format(model_path)
        out_dir = model_dir.replace('model_save','screen')
        if not os.path.exists(out_dir):
            os.makedirs(out_dir)
        if os.path.isdir(screen_file):
            p = mp.Pool(processes=4)
            for file_content in os.listdir(screen_file):
                file_path = os.path.join(screen_file, file_content)
                param = {'file': file_path, 'sep': sep, 'models': model_path, 'prop': prop, 'out_dir': out_dir}
                get = p.apply_async(ml_screener.cir_file, kwds=param)
            p.close()
            p.join()
        elif os.path.isfile(screen_file):
            ml_screener.cir_file(file=screen_file, sep=sep, models=model_path, prop=prop, out_dir=out_dir)
        else:
            print('What\'s this ?')
    elif model =='DNN':
        result_dir = model_dir.replace('model_save', 'result_save')
        result_file = os.path.join(os.path.join(result_dir, model), '_'.join([split, model, FP,'para.csv']))
        model_param = open(result_file, 'r').readlines()[1].replace(']','').replace('[','').replace('\"','').replace(' ','').split(',')[3:8]
        model_param = [param if len(param)< 5 else '{:5f}'.format(float(param))[:6] for param in model_param]
        model_path = []
        for file in os.listdir(os.path.join(model_dir, model)):
            count = 0
            for param in model_param:
                if param in file:
                    count += 1
            if count == len(model_param) :
                model_path.append(os.path.join(os.path.join(model_dir, model), file))
        if len(model_path) > 0:
            model_path = model_path[0]
            out_dir = model_dir.replace('model_save', 'screen')
            if not os.path.exists(out_dir):
                os.makedirs(out_dir)
            if os.path.isdir(screen_file):
                p = mp.Pool(processes=4)
                for file_content in os.listdir(screen_file):
                    file_path = os.path.join(screen_file, file_content)
                    param = {'file': file_path, 'sep': sep, 'models': model_path, 'prop': prop, 'out_dir': out_dir,'smiles_col':smiles_col}
                    get = p.apply_async(dnn_screener.screen, kwds=param)
                p.close()
                p.join()
            elif os.path.isfile(screen_file):
                dnn_screener.screen(file=screen_file, sep=sep, models=model_path, prop=prop, out_dir=out_dir,smiles_col=smiles_col)
            else:
                print('What\'s this ?')
    else:
        result_dir = model_dir.replace('model_save','result_save')
        result_file = os.path.join(os.path.join(result_dir,model),'_'.join([split,model,'para.csv']))
        model_param = open(result_file,'r').readlines()[1].replace("\"",'').replace(" ",'').split(',')[:-9]
        model_param = [i.replace(')','').replace('(','') if len(i)<15 else i[:6]  for i in model_param]
        model_path = []
        for file in os.listdir(os.path.join(model_dir,model)):
            count=0
            for param in model_param:
                if param in file:
                    count+=1
            if count==len(model_param):
                model_path.append(os.path.join(os.path.join(model_dir,model),file))
        if len(model_path)>0:
            model_path = model_path[0]
            out_dir = model_dir.replace('model_save', 'screen')
            if not os.path.exists(out_dir):
                os.makedirs(out_dir)
            if os.path.isdir(screen_file):
                p = mp.Pool(processes=4)
                for file_content in os.listdir(screen_file):
                    file_path = os.path.join(screen_file, file_content)
                    param = {'file': file_path, 'sep': sep, 'models': model_path, 'prop': prop, 'out_dir': out_dir,'smiles_col':smiles_col}
                    get = p.apply_async(dl_screener.screen, kwds=param)
                p.close()
                p.join()
            elif os.path.isfile(screen_file):
                dl_screener.screen(file=screen_file, sep=sep, models=model_path, prop=prop, out_dir=out_dir,smiles_col=smiles_col)
            else:
                print('What\'s this ?')




        else:
            assert len(model_path)==1, 'no such model file'




# model_screen(model=None,FP= None,split='cluster',screen_file='/data/jianping/web-ocaicm/bokey/BACE/BACE.csv',prop = 0.5,sep = ',',model_dir='/data/jianping/bokey/OCAICM/dataset/chembl31/model_save',smiles_col='Smiles')



from PyaiVS import model_bulid
model_bulid.running('/data/jianping/bokey/OCAICM/dataset/aurorab/aurorab.csv',out_dir='/data/jianping/bokey/OCAICM/dataset/',
                    split='all',model=['SVM','KNN','DNN','RF','XGB'],FP='all', run_type='result',cpus=6)


import os.path

from rdkit.Chem import AllChem

from rdkit.Chem.Scaffolds.MurckoScaffold import MurckoScaffoldSmiles
import numpy as np
from sklearn import preprocessing
from sklearn.metrics import roc_auc_score, confusion_matrix, precision_recall_curve, auc
import random
import pandas as pd

from rdkit import Chem
from rdkit import DataStructs
from rdkit.ML.Cluster import Butina
from rdkit.Chem import rdFingerprintGenerator
def saltremover(i):
    l = i.split('.')
    d = {len(a):a for a in l }
    smile = d[max(d.keys())]
    return smile
def stand_smiles(smiles):
    try:
        smiles = AllChem.MolToSmiles(AllChem.MolFromSmiles(smiles))
    except:
        smiles = ''
    return smiles
def process(file,content,cpu=10):
    if content == 'cano':
        data = pd.read_csv(file)
        start =len(data)
        data['Smiles'] = data['Smiles'].apply(saltremover)
        data['Smiles'] = data['Smiles'].apply(stand_smiles)
        output = file.split('.csv')[0] + '_pro.csv'
        if os.path.exists(output):
            pass
        else:
            data.to_csv(output,index=False)
            print('we meet some smiles which cannot revert to cano_smiles and the number is',start-len(data))
    if content == 'descriptor':
        from padelpy import padeldescriptor
        output = file.split('.csv')[0] + '_pro.csv'
        data = pd.read_csv(output)
        data['activity'] = data['Smiles']
        data=data[['Smiles','activity']]
        smi = file.split('.csv')[0] +'.smi'
        des = file.split('.csv')[0] +'_23d.csv'
        if os.path.exists(des):
            pass
        else:
            data.to_csv(smi,index=False,sep='\t',header=None)
            padeldescriptor(mol_dir=smi, d_2d=True, d_3d=True, d_file=des,threads=50)
            # print('done 2d3d',end=' ')
    if content == 'pubchem':
        from padelpy import padeldescriptor
        output = file.split('.csv')[0] + '_pro.csv'
        data = pd.read_csv(output)
        data['activity'] = data['Smiles']
        data=data[['Smiles','activity']]
        smi = file.split('.csv')[0] +'.smi'
        des = file.split('.csv')[0] +'_pubchem.csv'
        if os.path.exists(des):
            pass
        else:
            data.to_csv(smi,index=False,sep='\t',header=None)
            padeldescriptor(mol_dir=smi, fingerprints=True, d_file=des,threads=30)
            # print('done punchem',end=' ')
    if content == 'adj_23d':
        des = file.split('.csv')[0] + '_23d.csv'
        name = file.split('.csv')[0] + '_23d_adj.csv'
        if os.path.exists(name):
            pass
        else:
            des = pd.read_csv(des).iloc[:,:1445]   # save pre 1444 features
            des = des.replace([np.inf,-np.inf],np.nan)  # trans inf to nan
            des = des.dropna(thresh=int(des.shape[1]/2),axis =0)  #drop that row all 0 how ='all'
            des = des.dropna(thresh=int(des.shape[0]/2),axis =1)
            des = des.fillna(0)  # fill nan by mean col
            min_max_scaler = preprocessing.MinMaxScaler()
            adj = min_max_scaler.fit_transform(des.drop(['Name'], axis=1))  # scaler
            adj = pd.DataFrame(adj,columns=list(des.columns)[1:])
            adj['Name']=des['Name']
            adj.to_csv(name,index=False)
            # print('done scaler',end =' ')
def start(file,des= None):
    content = ['cano']
    if '2d-3d' in des:
        content.extend(['descriptor','adj_23d'])
    if 'pubchem' in des:
        content.append('pubchem')
    for pro in content:
        process(file,pro)
def statistical(y_true, y_pred, y_pro):
    c_mat = confusion_matrix(y_true, y_pred)
    tn, fp, fn, tp = list(c_mat.flatten())
    se = tp / (tp + fn)
    sp = tn / (tn + fp)
    precision = tp / (tp + fp)
    acc = (tp + tn) / (tn + fp + fn + tp)
    mcc = (tp * tn - fp * fn) / np.sqrt((tp + fp) * (tp + fn) * (tn + fp) * (tn + fn) + 1e-8)
    auc_prc = auc(precision_recall_curve(y_true, y_pro, pos_label=1)[1],
                  precision_recall_curve(y_true, y_pro, pos_label=1)[0])
    auc_roc = roc_auc_score(y_true, y_pro)
    return precision, se, sp, acc, mcc, auc_prc, auc_roc




class TVT(object):
    def __init__(self,X,Y):
        self.X = X
        self.Y = Y
        self.column = pd.Series(data=self._index(len(X)),index=self.X.index)
    def __len__(self):
        return len(self.X)
    def __getitem__(self, idx):
        X = self.X[idx]
        Y = self.Y[idx]
        return X, Y
    def _index(self,number):
        span = number//10 +1
        end = number - 9*span
        column = []
        for num in range(10):
            if num != 9:
                column.extend([num]*span)
            else:
                column.extend([num] * end)
        random.seed(42)
        random.shuffle(column)
        return column
    def set2ten(self,num):
        assert 0<=num<=9,'num [0,9]'
        test_X = self.X.iloc[self.column[self.column==num].index,]
        test_Y = self.Y.iloc[self.column[self.column == num].index,]
        rest_X = self.X.iloc[self.column[self.column != num].index,]
        rest_Y = self.Y.iloc[self.column[self.column != num].index,]
        # in this .py ,we don't add a check to confirm if all one class in r/test_Y
        return rest_X, test_X, rest_Y,test_Y

class ScaffoldSplitter():


    def _generate_scaffolds(self,dataset):
        scaffolds = {}
        for i,ind in enumerate(list(dataset.index)):
            smiles = dataset.iloc[i]
            scaffold = self._generate_scaffold(smiles)
            if scaffold not in scaffolds:
                scaffolds[scaffold] = [ind]
            else:
                scaffolds[scaffold].append(ind)

    # Sort from largest to smallest scaffold sets
        scaffolds = {key: sorted(value) for key, value in scaffolds.items()}
        scaffold_sets = [
                scaffold_set for (scaffold, scaffold_set) in sorted(
                scaffolds.items(), key=lambda x: (len(x[1]), x[1][0]), reverse=True)
            ]
        return scaffold_sets

    def _generate_scaffold(self, smiles):

        mol = Chem.MolFromSmiles(smiles)
        scaffold = MurckoScaffoldSmiles(mol=mol)
        return scaffold

    def train_test_split(self,X,Y,frac_train=0.8,valid=False):
        frac_valid=(1-frac_train)/2 if valid else (1-frac_train)
        scaffold_sets = self._generate_scaffolds(X)
        train_cutoff = frac_train * len(X)
        valid_cutoff = (frac_train + frac_valid) * len(X)
        train_inds,valid_inds,test_inds = [],[],[]
        for scaffold_set in scaffold_sets:
            if len(train_inds) + len(scaffold_set) > train_cutoff:
                if len(train_inds) + len(valid_inds) + len(scaffold_set) > valid_cutoff:
                    test_inds += scaffold_set
                else:
                    valid_inds += scaffold_set
            else:
                train_inds += scaffold_set
        if valid:
            X_train,Y_train = X.loc[train_inds,],Y.loc[train_inds,]
            X_valid,Y_valid =X.loc[valid_inds,],Y.loc[valid_inds,]
            X_test,Y_test=X.loc[test_inds,],Y.loc[test_inds,]
            return X_train,X_valid,X_test,Y_train,Y_valid,Y_test
        else:
            X_train, Y_train = X.loc[train_inds,], Y.loc[train_inds,]
            X_valid, Y_valid = X.loc[valid_inds,], Y.loc[valid_inds,]
            return X_train, X_valid, Y_train, Y_valid

class ClusterSplitter():

    def _tanimoto_distance_matrix(self,fp_list):
        """Calculate distance matrix for fingerprint list"""
        dissimilarity_matrix = []

        for i in range(1, len(fp_list)):
            # Compare the current fingerprint against all the previous ones in the list
            similarities = DataStructs.BulkTanimotoSimilarity(fp_list[i], fp_list[:i])
            # Since we need a distance matrix, calculate 1-x for every element in similarity matrix
            dissimilarity_matrix.extend([1 - x for x in similarities])
        return dissimilarity_matrix

    def _cluster_fingerprints(self,fingerprints, cutoff=0.2):
        distance_matrix = self._tanimoto_distance_matrix(fingerprints)
        clusters = Butina.ClusterData(distance_matrix, len(fingerprints), cutoff, isDistData=True)
        clusters = sorted(clusters, key=len, reverse=True)
        return clusters
    def _generate_cluster(self, smiles):

        mol = Chem.MolFromSmiles(smiles)
        scaffold = MurckoScaffoldSmiles(mol=mol)
        return scaffold
    def _generate_clusters(self,dataset,train_size=0.8):
        fingerprints=[]
        rdkit_gen = rdFingerprintGenerator.GetRDKitFPGenerator(maxPath=5)
        for i,ind in enumerate(list(dataset.index)):
            smiles = dataset.iloc[i]
            fingerprints.append(rdkit_gen.GetFingerprint(Chem.MolFromSmiles(smiles)))
        clusters = self._cluster_fingerprints(fingerprints, cutoff=0.2)
        cluster_centers = [c[0] for c in clusters]
        sorted_clusters = []
        Singletons = []
        for cluster in clusters:
            if len(cluster) <= 1:
                Singletons.append(cluster)
                continue  # Singletons
            sorted_fingerprints = [rdkit_gen.GetFingerprint(Chem.MolFromSmiles(dataset.loc[i,])) for i in cluster]
            similarities = DataStructs.BulkTanimotoSimilarity(
                sorted_fingerprints[0], sorted_fingerprints[1:]
            )
            similarities = list(zip(similarities, cluster[1:]))
            similarities.sort(reverse=True)
            sorted_clusters.append((len(similarities), [i for _, i in similarities]))
            sorted_clusters.sort(reverse=True)
        selected_molecules = cluster_centers.copy()
        index = 0
        pending = int(len(dataset) * train_size) - len(selected_molecules)
        while pending > 0 and index < len(sorted_clusters):
            tmp_cluster = sorted_clusters[index][1]
            if sorted_clusters[index][0] > 10:
                num_compounds = int(sorted_clusters[index][0] * train_size)
            else:
                num_compounds = len(tmp_cluster)
            if num_compounds > pending:
                num_compounds = pending
            selected_molecules += [i for i in tmp_cluster[:num_compounds]]
            index += 1
            pending = int(len(dataset) * train_size) - len(selected_molecules)
        return selected_molecules

    def train_test_split(self,X,Y,frac_train=0.8,valid=False):
        train_inds = self._generate_clusters(X,train_size=frac_train)
        if valid:
            rest_inds = list(set(X.index) - set(train_inds))
            valid_inds = rest_inds[:int(len(rest_inds)/2)]
            test_inds = rest_inds[int(len(rest_inds)/2):]
            X_train, Y_train = X.loc[train_inds,], Y.loc[train_inds,]
            X_valid, Y_valid = X.loc[valid_inds,], Y.loc[valid_inds,]
            X_test, Y_test = X.loc[test_inds,], Y.loc[test_inds,]
            return X_train, X_valid, X_test, Y_train, Y_valid, Y_test
        else:
            valid_inds = list(set(X.index)-set(train_inds))
            X_train, Y_train = X.loc[train_inds,], Y.loc[train_inds,]
            X_valid, Y_valid = X.loc[valid_inds,], Y.loc[valid_inds,]
            return X_train, X_valid, Y_train, Y_valid

# file = '/data/jianping/bokey/OCAICM/dataset/aurorab/aurorab.csv'
# df = pd.read_csv(file)
# sp = ClusterSplitter()
# X_train, X_valid,c, Y_train, Y_valid,d = sp.train_test_split(df['Smiles'],df['activity'],frac_train=0.8,valid=True)
# print(X_valid, Y_valid)


import pandas as pd
import numpy as np
from rdkit import Chem
from rdkit.Chem import AllChem
from rdkit.Chem import Lipinski, Descriptors, Crippen
import multiprocessing as mp
from rdkit.Chem.FilterCatalog import FilterCatalog, FilterCatalogParams
import argparse
import joblib
import time
import os
s = time.time()
params = FilterCatalogParams()
params.AddCatalog(FilterCatalogParams.FilterCatalogs.PAINS)
catalog = FilterCatalog(params)
def ro5(x):
    try:
        mol = Chem.MolFromSmiles(x)
        h_acc = Lipinski.NumHAcceptors(mol)
        h_don = Lipinski.NumHDonors(mol)
        # rotal = Lipinski.NumRotatableBonds(mol)
        weight = Descriptors.ExactMolWt(mol)
        logp = Crippen.MolLogP(mol)
        count = sum([weight <= 500, h_acc <= 10, h_don <= 5, logp <= 5])
        return 1 if count >= 3 else 0
    except:
        return 0
def pains(x):
    mol = Chem.MolFromSmiles(x)
    entry = catalog.GetFirstMatch(mol)  # Get the first matching PAINS
    return 1 if entry is not None else 0
def fp(X, type='ECFP4',file=''):
    smiles = np.array(X)
    count = 0
    if type == 'ECFP4':
        ms = [Chem.MolFromSmiles(smiles[i]) for i in range(len(smiles))]
        ecfpMat = np.zeros((len(ms), 1024), dtype=int)
        for i in range(len(ms)):
            try:
                fp = AllChem.GetMorganFingerprintAsBitVect(ms[i], 2, 1024)
                ecfpMat[i] = np.array(list(fp.ToBitString()))
            except:
                count += 1
                ecfpMat[i] = np.zeros((1, 1024), dtype=int)
        X = ecfpMat

        return X
    elif type == 'MACCS':
        ms = [Chem.MolFromSmiles(smiles[i]) for i in range(len(smiles))]
        ecfpMat = np.zeros((len(ms), 167), dtype=int)
        for i in range(len(ms)):
            fp = AllChem.GetMACCSKeysFingerprint(ms[i])
            ecfpMat[i] = np.array(list(fp.ToBitString()))
        X = ecfpMat
        return X
    elif type == 'pubchem':
        des = pd.read_csv(file)
        des = des.set_index('Name')
        features = len(des.columns)
        ecfpMat = np.zeros((len(X), features), dtype=int)
        for j, smile in enumerate(smiles):
            index = str(smile)
            try:
                ecfpMat[j] = np.array(des.loc[index, :])
            except:
                pass
        X = ecfpMat
        return X

def parse_args():
    parser = argparse.ArgumentParser()
    parser.add_argument('--file', required=True, help='we must give this para')
    parser.add_argument('--models', required=True, nargs='*')
    parser.add_argument('--prop', default=0.5, type=float)
    parser.add_argument('--sep', default=',', type=str)
    parser.add_argument('--smiles_col', default='smiles', type=str)
    parser.add_argument('--cpus', default=1,type=int)
    parser.add_argument('--out_dir')
    args = parser.parse_args()
    return args
def cir_file(file=None,sep=',',models=None,prop=0.5,out_dir='./',smiles_col='Smiles'):
    count = 0
    com = 0
    out_file = os.path.join(out_dir,file.split('/')[-1].replace('.csv','_screen_{}.csv'.format(prop)))
    f = open(out_file,'w')
    assert smiles_col in open(file,'r').readline().split(sep) ,'{} is not in screen_file columns'.format(smiles_col)
    local  = open(file,'r').readline().split(sep).index(smiles_col)
    print(local)
    for line in open(file,'r'):

        if line.startswith('smiles') or line.startswith('Smiles'):

            continue
        com +=1
        smiles = line.split(sep)[local]
        model = models
        type = model.split('_')[-3]
        fprint = fp([smiles],type=type,file=file)
        model = joblib.load(model)
        pred = model.predict_proba(fprint)
        if pred[0][0]>=prop:
            lpsk = ro5(smiles)
            if lpsk==1:
                count +=1
                f.write('{}\n'.format(smiles))
        if count/10000==0:
            print(count)
    f.close()
    print('screen precent ',round((count/com),2))

if __name__ == '__main__':
    args = parse_args()
    file = args.file
    models = args.models
    sep = args.sep
    prop = args.prop
    smiles_col = args.smiles_col
    cpus = args.cpus
    out_dir = args.out_dir

    if os.path.isdir(file):
        p = mp.Pool(processes=cpus)
        for file_content in os.listdir(file):
            if '.csv' in file_content and 'pubchem' not in file_content:
                file_path = os.path.join(file,file_content)
                param = {'file':file_path, 'sep':sep, 'models':models, 'prop':prop, 'out_dir':out_dir}
                get = p.apply_async(cir_file,kwds = param)
        p.close()
        p.join()
    elif os.path.isfile(file):
        cir_file(file=file, sep=sep, models=models, prop=prop, out_dir=out_dir)
    else:
        print('What\'s this ?')



import torch
import os
import numpy as np
import pandas as pd
from PyaiVS.dnn_torch_utils import Meter, MyDataset, EarlyStopping, MyDNN, collate_fn, set_random_seed
from hyperopt import fmin, tpe, hp, rand, STATUS_OK, Trials, partial
from torch.utils.data import DataLoader
from torch.nn import BCEWithLogitsLoss, MSELoss
import gc
import time
start_time = time.time()
import warnings
from sklearn import preprocessing
from bokeys.splitdater import split_dataset
from bokeys.feature_create import create_des
import torch
torch.set_num_threads(5)

def run_an_eval_epoch(model, data_loader, args):
    f = open(args['output'],'w+')
    f.write('cano_smiles,pred_prop\n')
    count = 0
    model.eval()
    # eval_metric = Meter()
    with torch.no_grad():
        for batch_id, batch_data in enumerate(data_loader):
            eval_metric = Meter()
            Xs, Ys, masks = batch_data
            Xs, Ys, masks = Xs.to(args['device']), Ys.to(args['device']), masks.to(args['device'])
            outputs = model(Xs)
            outputs.cpu()
            Ys.cpu()
            masks.cpu()
#            torch.cuda.empty_cache()

            eval_metric.update(outputs, Ys, torch.tensor([count]))
            roc_score = eval_metric.compute_metric('pred')
            smiles = args['data'][args['smiles_col']].tolist()[int(Ys[0])]
            write_check = 0
            for score in roc_score:
                if score>= args['prop']:
                    write_check =1
                    break
            if write_check ==1:

                f.write('{},{}\n'.format(smiles, ','.join([str(round(float(score),3)) for score in roc_score])))
            count += 1
            torch.cuda.empty_cache()
        f.close()
def screen(file='', sep=',', models=None,prop=0.5, smiles_col='Smiles',out_dir=None,tasks=1):

    my_df = pd.read_csv(file, engine='python', sep=sep)

    device = torch.device("cpu")
    args = {'device': device, 'metric': 'roc_auc','prop':prop,'data':my_df,'smiles_col':smiles_col,'tasks':tasks}
    outputs = os.path.join(out_dir,
                           file.split('/')[-1].replace('.csv', '_screen_{}_{}.csv'.format(args['prop'], 'DNN')))
    if os.path.exists(outputs):
        print(outputs, 'has done')
    else:
        args['output'] = outputs
        FP_type = models.split('/')[-1].split('_')[1]
        model_dir = out_dir.replace(out_dir.split('/')[-1],'model_save')
        print(smiles_col)
        data_x, data_y = create_des(my_df[smiles_col], list(range(len(my_df))), FP_type=FP_type, model_dir=model_dir)

        dataset = MyDataset(data_x, data_y)
        loader = DataLoader(dataset,  collate_fn=collate_fn)
        inputs = data_x.shape[1]
        hideen_unit = (eval(models.split('/')[-1].split('_')[5]),
                       eval(models.split('/')[-1].split('_')[6])
                       ,eval(models.split('/')[-1].split('_')[7]))

        dropout = eval(models.split('/')[-1].split('_')[4])
        best_model = MyDNN(inputs=inputs, hideen_units=hideen_unit, outputs=tasks,
                           dp_ratio=dropout, reg=False)
        best_model.load_state_dict(torch.load(models, map_location=device)['model_state_dict'])
        best_model.to(device)
        run_an_eval_epoch(best_model, loader, args)
#
screen(models ='/data/jianping/bokey/OCAICM/dataset/chembl31/model_save/DNN/cla_ECFP4_cluster_dataset_0.0016_64_512_128_0.0047_early_stop.pth',
       file='/data/jianping/bokey/OCAICM/dataset/chembl31/model_save/BACE_screen_0.5_DNN.csv',prop = 0.5,sep = ',',
       out_dir='/data/jianping/bokey/OCAICM/dataset/chembl31/model_save',smiles_col='cano_smiles',tasks=1)


import setuptools

setuptools.setup(
        name='PyaiVS',
        version='1.1.1',
        description="model_bulid & vitual_screen",
        # long_description=open('README.md').read(),
        include_package_data=True,
        author='bokey',
        author_email='',
        url='',
        classifiers=[
            'Development Status :: 3 - Alpha',
            'Intended Audience :: Developers',
            'Topic :: Software Development :: Build Tools',
            'License :: OSI Approved :: MIT License',
            'Programming Language :: Python :: 3',
        ],
        python_requires='>=3',
        #install_requires=['rdkit==2022.03.5','pytorch==1.9.0','dgl-cuda10.2==0.4.3post2',
        #                  'deepchem==2.6.1','xgboost==1.5.1','hyperopt==0.2.7','pytorch-gpu==1.9.0','torchaudio==0.9.0 '],  # 安装所需要的库
        entry_points={
            'console_scripts': [
                ''],
        },
)



import os
import pandas as pd
from dgl.data.chem import csv_dataset, smiles_to_bigraph, MoleculeCSVDataset
from PyaiVS.gnn_utils import AttentiveFPBondFeaturizer, AttentiveFPAtomFeaturizer, collate_molgraphs, \
     Meter
from torch.utils.data import DataLoader
from dgl.model_zoo.chem import MPNNModel, GCNClassifier, GATClassifier, AttentiveFP
import multiprocessing as mp
from rdkit.Chem import AllChem
from rdkit.Chem import rdmolfiles
import os
import time
s =time.time()


import torch
torch.set_num_threads(5)
def run_an_eval_epoch(model, data_loader, args):
    f = open(args['output'],'w+')

    f.write('cano_smiles,pred_prop\n')
    # print(args['output'])
    model.eval()
    # eval_metric = Meter()
    smile_list = {}
    count  = 0
    with torch.no_grad():
        for batch_id, batch_data in enumerate(data_loader):
            eval_metric = Meter()
            smiles, bg, labels, masks = batch_data
            smile_list[count]=smiles
            atom_feats = bg.ndata.pop('h')
            bond_feats = bg.edata.pop('e')
            # transfer the data to device(cpu or cuda)
            outputs = model(bg, atom_feats) if args['model'] in ['gcn', 'gat'] else model(bg, atom_feats,bond_feats)
            # smile_list.append(smiles)
            eval_metric.update(outputs, labels, torch.tensor([count]))
            roc_score = eval_metric.compute_metric('pred')
            if roc_score.tolist()[0][0] >= args['prop']:
                f.write('{},{}\n'.format(smiles[0],round(roc_score.tolist()[0][0],2)))
                # print(smiles[0],round(roc_score.tolist()[0][0],2))
            count += 1
            torch.cuda.empty_cache()
        f.close()
#file=screen_file, sep=sep,  prop=prop, out_dir=out_dir
def screen(file='', sep=',', models=None,prop=0.5, smiles_col='Smiles',out_dir=None):
    AtomFeaturizer = AttentiveFPAtomFeaturizer
    BondFeaturizer = AttentiveFPBondFeaturizer
    print(models)
    model_type = models.split('/')[-2]
    args = {'model': model_type,'prop':prop}
    outputs = os.path.join(out_dir,file.split('/')[-1].replace('.csv','_screen_{}_{}.csv'.format(args['prop'],args['model'])))
    print(outputs)
    if os.path.exists(outputs):
        print(outputs,'has done')
    else:
        args['output'] =outputs
        my_df = pd.read_csv(file,engine='python',sep=sep)
        print(my_df.columns)
        my_dataset: MoleculeCSVDataset = csv_dataset.MoleculeCSVDataset(my_df.iloc[:, :], smiles_to_bigraph, AtomFeaturizer,
                                                                            BondFeaturizer, smiles_col,
                                                                            file.replace('.csv', '.bin'))
        train_loader = DataLoader(my_dataset,shuffle=True,batch_size=1,
                                      collate_fn=collate_molgraphs)
        if 'gcn' in models:
            chf = models.split('_')[-1].split('.')[0]
            ghf = models.split('_')[-2]
            best_model = GCNClassifier(in_feats=AtomFeaturizer.feat_size('h'),
                                               gcn_hidden_feats=eval(ghf),
                                               n_tasks=1,
                                               classifier_hidden_feats=eval(chf))

        elif 'gat' in models:
            chf = models.split('_')[-1].split('.')[0]
            nh = models.split('_')[-2]
            ghf = models.split('_')[-3]

            best_model = GATClassifier(in_feats=AtomFeaturizer.feat_size('h'),
                                       gat_hidden_feats=eval(ghf),
                                       num_heads=eval(nh), n_tasks=1,
                                       classifier_hidden_feats=eval(chf))
        elif 'attentivefp' in models:
            num_layer = eval(models.split('_')[-4])
            num_timestep  = eval(models.split('_')[-3])
            graph_feat_size = eval(models.split('_')[-2])
            dropout = eval(models.split('_')[-1].split('.')[0])
            best_model = AttentiveFP(node_feat_size=AtomFeaturizer.feat_size('h'),
                                     edge_feat_size=BondFeaturizer.feat_size('e'),
                                     num_layers=num_layer,
                                     num_timesteps=num_timestep,
                                     graph_feat_size=graph_feat_size,
                                     output_size=1,
                                     dropout=dropout)
        else:
            node_hidden_dim =eval(models.split('_')[-3])
            edge_hidden_dim = eval(models.split('_')[-2])
            num_layer_set2set = eval(models.split('_')[-1].split('.')[0])
            best_model = MPNNModel(node_input_dim=AtomFeaturizer.feat_size('h'),
                                   edge_input_dim=BondFeaturizer.feat_size('e'),
                                   output_dim=1,
                                   node_hidden_dim=node_hidden_dim,
                                   edge_hidden_dim=edge_hidden_dim,
                                   num_layer_set2set=num_layer_set2set)
        best_model.load_state_dict(torch.load(models, map_location='cpu')['model_state_dict'])
        run_an_eval_epoch(best_model, train_loader, args)

# if __name__ == '__main__':
#     model_path ='/data/jianping/bokey/library/dataset/GLS1/model_save/gcn/gcn_scaffold_GLS1_0_0.031623_(256, 256)_128.pth'
#     dir = '/data/jianping/web-ocaicm/bokey/BACE/BACE.csv'
#     output = '/data/jianping/bokey/library/dataset/GLS1/screen'
#     if not os.path.exists(output):
#         os.makedirs(output)
#     file = dir
#     if os.path.isdir(file):
#         p = mp.Pool(processes=6)
#         #
#         for file_content in os.listdir(file):
#             if '.csv' in file_content:
#                 file_path = os.path.join(file,file_content)
#                 param = {'file':file_path,  'prop':0.75, 'smiles_col':'smiles','output':output}
#                 get = p.apply_async(screen,kwds = param)
#         p.close()
#         p.join()
#     elif os.path.isfile(file):
#         screen(file=file,prop=0.5,smiles_col='Smiles',output =output)
#     else:
#         pass
#     # file_list = ['/data/jianping/bokey/library/database/total_db_csv/NC_320593.csv',
#     #              '/data/jianping/bokey/library/database/total_db_csv/Express_Pick _20mg_01_2019_774293_Part2.csv',
#     #              '/data/jianping/bokey/library/database/total_db_csv/Express_Pick _20mg_01_2019_774293_Part1.csv',
#     #              '/data/jianping/bokey/library/database/total_db_csv/DC03_239319.csv',
#     #              '/data/jianping/bokey/library/database/total_db_csv/DC02_350000.csv',
#     #              '/data/jianping/bokey/library/database/total_db_csv/DC01_350000.csv']
#
#     # def mol_check(smiles):
#     #     try:
#     #         mol = AllChem.MolFromSmiles(smiles)
#     #         rdmolfiles.CanonicalRankAtoms(mol)
#     #         return 1
#     #     except:
#     #         return 0
#     # def datacheck(file):
#     #
#     #     my_df = pd.read_csv(file, sep=';', usecols=[i for i in range(5)], engine='python')
#     #     my_df.columns = open(file, 'r').readline().split(';')[:5]
#     #     # print('pass')
#     #     # my_df.columns = [smiles_col]
#     #     # my_df = pd.read_csv(file,sep=';')
#     #     my_df['index'] = my_df.index
#     #     my_df = my_df[['smiles', 'index']]
#     #     my_df['check'] = my_df['smiles'].apply(mol_check)
#     #     my_df =my_df[my_df['check']==1]
#     #     output =file.replace('.csv','_graph.csv')
#     #     my_df.to_csv(output,index=False)
#     #
#     #
#     # p = mp.Pool(processes=6)
#     # for file in file_list:
#     #     file =file.replace('.csv','_graph.csv')
#     #     screen(file=file,prop=0.5,smiles_col='smiles')
#     # p.close()
#     # p.join()